@@ -13,15 +13,16 @@
 # testing and support for custom GSM commands, so use it at your own risk,
 # and without ANY warranty! Have fun.
 #
-# $Id: Gsm.pm,v 1.34 2004/08/18 07:07:32 cosimo Exp $
+# $Id: Gsm.pm,v 1.37 2005/08/27 12:45:24 cosimo Exp $
 
 package Device::Gsm;
-$Device::Gsm::VERSION = sprintf "%d.%02d", q$Revision: 1.34 $ =~ /(\d+)\.(\d+)/;
+$Device::Gsm::VERSION = '1.37';
 
 use strict;
 use Device::Modem;
 use Device::Gsm::Sms;
 use Device::Gsm::Pdu;
+use Device::Gsm::Charset;
 
 @Device::Gsm::ISA = ('Device::Modem');
 
@@ -128,7 +129,7 @@ sub delete_sms {
     my $msg_index = $_[1];
     my $ok;
 
-    if( ! defined $msg_index || ! $msg_index ) {
+    if( ! defined $msg_index || $msg_index eq '' ) {
         $self->log->write('warn', 'undefined message number. cannot delete sms message');
         return 0;
     }
@@ -218,7 +219,7 @@ sub model() {
 		$self->atsend( 'AT+CGMM' . Device::Modem::CR );
 		($code, $model) = $self->parse_answer();
 
-		$self->log->write('info', 'model of this device is ['.$model.']');
+		$self->log->write('info', 'model of this device is ['.($model||'').']');
 
 	}
 
@@ -544,10 +545,14 @@ sub _send_sms_text {
 	# Send sms in text mode
 	$me->atsend( qq[AT+CMGS="$num"] . Device::Modem::CR );
 
+	# Encode text
+	$text = Device::Gsm::Charset::iso8859_to_gsm0338( $text );
+
+	# Complete message sending
 	$me->atsend( $text . Device::Modem::CTRL_Z );
 
 	# Get reply and check for errors
-	$cReply = $me->answer('+CMGS', 2000);
+	$cReply = $me->answer('\+CMGS', 2000);
 	if( $cReply =~ /ERROR/i ) {
 		$me->log->write( 'warning', "ERROR in sending SMS" );
 	} else {
@@ -578,6 +583,13 @@ sub _send_sms_pdu {
 	# TODO Validity period (now fixed to 4 days)
 	my $vp = 'AA';
 
+	# Status report requested?
+	my $status_report = 0;
+	if( exists $opt{'status_report'} && $opt{'status_report'} )
+	{
+		$status_report = 1;
+	}
+
 	my $lOk = 0;
 	my $cReply;
 
@@ -597,11 +609,22 @@ sub _send_sms_pdu {
 	$me->log->write('info', 'encoded dest. address is ['.$enc_da.']');
 
 	# Encode text
+	$text = Device::Gsm::Charset::iso8859_to_gsm0338( $text );
 	my $enc_msg = Device::Gsm::Pdu::encode_text7( $text );
 	$me->log->write('info', 'encoded 7bit text (w/length) is ['.$enc_msg.']');
 
 	# Build PDU data
-	my $pdu = uc join( '', '00', '11', '00', $enc_da, '00', $class, $vp, $enc_msg );
+	my $pdu = uc join(
+		'',
+		'00',
+		($status_report ? '31' : '11'),
+		'00',
+		$enc_da,
+		'00',
+		$class,
+		$vp,
+		$enc_msg
+	);
 
 	$me->log->write('info', 'due to send PDU ['.$pdu.']');
 
@@ -690,174 +713,6 @@ sub service_center(;$) {
 
 }
 
-# Transform ascii char set to gsm 3.38 charset
-{
-
-	my @gsm = map chr, 0 .. 255;
-	$gsm[0] = '@';
-	$gsm[1] = '';
-	$gsm[2] = '$';
-	$gsm[3] = '';
-	$gsm[5] = '';
-	$gsm[4] = '';
-	$gsm[6] = '';
-	$gsm[7] = '';
-	$gsm[8] = '';
-	$gsm[9] = '';
-	$gsm[11] = '';
-	$gsm[12] = $gsm[11];
-	$gsm[15] = '';
-	$gsm[17] = '_';
- 	$gsm[20] = '^';
- 	$gsm[27] = chr(164); # '';
-	$gsm[29] = '';
-	$gsm[30] = chr(223); # '';
-	$gsm[31] = chr(201); # '';
-	$gsm[36] = '';
-# 	$gsm[47] = '\\';
-#	$gsm[60] = '[';
-#	$gsm[62] = ']';
- 	$gsm[92] = '/';
- 	$gsm[95] = '';
-	$gsm[123] = '';
-	$gsm[127] = '';
-	$gsm[124] = '';
-	$gsm[125] = '';
-	$gsm[126] = '';
-	$gsm[164] = '';
- 	$gsm[232] = '';
- 	$gsm[233] = '';
- 	$gsm[236] = '';
- 	$gsm[248] = '';
-
-=cut
-
-CODE 	$gsm[95] = '';
-CODE 	$gsm[27] = '';
-CODE 	$gsm[101] = '';
-CODE 	$gsm[5] = '';
-CODE 	$gsm[4] = 'g';
-CODE 	$gsm[103] = 'h';
-CODE 	$gsm[104] = 'i';
-CODE 	$gsm[105] = '4';
-CODE 	$gsm[52] = '';
-CODE 	$gsm[7] = '[';
-CODE 	$gsm[27] = '\';
-CODE 	$gsm[60] = ']';
-CODE 	$gsm[27] = '^';
-CODE 	$gsm[47] = 'j';
-CODE 	$gsm[27] = 'k';
-CODE 	$gsm[62] = 'l';
-CODE 	$gsm[27] = '5';
-CODE 	$gsm[20] = 'm';
-CODE 	$gsm[106] = 'n';
-CODE 	$gsm[107] = 'o';
-CODE 	$gsm[108] = '6';
-CODE 	$gsm[53] = '';
-CODE 	$gsm[109] = '';
-CODE 	$gsm[110] = '';
-CODE 	$gsm[111] = '';
-CODE 	$gsm[54] = 'p';
-CODE 	$gsm[125] = 'q';
-CODE 	$gsm[8] = 'r';
-CODE 	$gsm[12] = 's';
-CODE 	$gsm[124] = '7';
-CODE 	$gsm[112] = 't';
-CODE 	$gsm[113] = 'u';
-CODE 	$gsm[114] = 'v';
-CODE 	$gsm[115] = '8';
-CODE 	$gsm[55] = '';
-CODE 	$gsm[116] = '';
-CODE 	$gsm[117] = 'w';
-CODE 	$gsm[118] = 'x';
-CODE 	$gsm[56] = 'y';
-CODE 	$gsm[6] = 'z';
-CODE 	$gsm[126] = '9';
-CODE 	$gsm[119] = '0';
-CODE 	$gsm[120] = '+';
-CODE 	$gsm[121] = '';
-CODE 	$gsm[122] = '';
-CODE 	$gsm[57] = '';
-CODE 	$gsm[48] = '';
-CODE 	$gsm[43] = '';
-
-=cut
-
-=cut
-
-	# The following is the GSM 3.38 standard charset, as shown
-	# on some Siemens documentation found on the internet
-	my $gsm_charset = join('',
-		'@$'."\n".''."\r".'',  # 16
-		'߻',
-		' !"# %&()*+,-./',
-		'0123456789:;<=>?',
-		'-ABCDEFGHIJKLMNO',
-		'PQRSTUVWXYZֻܻ',
-		'abcdefghijklmno',
-		'pqrstuvwxyz'
-	);
-
-=cut
-
-	my $gsm_charset = join('',@gsm);
-
-sub _ascii2gsm {
-	my $self = shift;
-	my $ascii = shift;
-
-	return '' unless $ascii;
-
-	my $gsm = '';
-	my $n = 0;
-	for( ; $n < length($ascii) ; $n++ ) {
-		$gsm .= chr index($gsm_charset, substr($ascii, $n, 1));
-	}
-
-	return $gsm;
-}
-
-sub _gsm2ascii {
-	my $self = shift;
-	my $gsm = shift;
-	return '' unless $gsm;
-
-	my $ascii = '';
-	my $n = 0;
-
-	for( ; $n < length($gsm) ; $n++ ) {
-
-		my $c = ord( substr( $gsm, $n, 1 ) );
-
-		# Extended charset ?
-		if( $c == 0x1B ) {                          # "escape extended mode"
-			$n++;
-			$c = ord(substr($gsm, $n, 1));
-			if( $c == 0x65 ) {                  # 'e'
-				$ascii .= chr(164);         # iso_8859_15 EURO SIGN
-			} elsif( $c == 0x14 ) {
-				$ascii .= '^';
-			} elsif( $c == 0x3C ) {
-				$ascii .= '[';
-			} elsif( $c == 0x2F ) {
-				$ascii .= '\\';
-			} elsif( $c == 0x3E ) {
-				$ascii .= ']';
-			} else {
-				$ascii .= chr($c);          # Un-managed "extended" chars
-			}
-		} else {
-			# Standard GSM 3.38 encoding
-			$ascii .= substr( $gsm_charset, $c, 1 );
-		}
-	}
-
-	return $ascii;
-}
-
-}
-
-
 
 2703;
 
@@ -1155,6 +1010,17 @@ It is just a matter of trying.
 
 Phone number of message recipient
 
+=item C<status_report>
+
+If present with a true value, it enables sending of SMS messages (only for PDU mode,
+text mode SMS won't be influenced by this parameter) with the status report,
+also known as delivery report, that is a short message that reports the status
+of your sent message.
+Usually this is only available if your mobile company supports this feature,
+and probably you will be charged a small amount for this service.
+
+More information on this would be welcome.
+
 =back
 
 =head2 service_center()
